// main.m (this is -*- ObjC -*-)
// 
// \author: Björn Giesler <giesler@ira.uka.de>
// 
// 
// 
// $Author: bjoern $
// $Locker:  $
// $Revision: 1.1.1.1 $
// $Date: 2004/02/14 18:00:04 $

/* system includes */
#include <AppKit/AppKit.h>

/* my includes */
/* (none) */

int main(int argc, const char **argv)
{
  return NSApplicationMain(argc, argv);
}
