// AddressView.h (this is -*- ObjC -*-)
// 
// \author: Björn Giesler <giesler@ira.uka.de>
// 
// Address View Framework for GNUstep
// 
// $Author: bjoern $
// $Locker:  $
// $Revision: 1.1.1.1 $
// $Date: 2004/02/14 18:00:01 $

#ifndef ADDRESSVIEW_H
#define ADDRESSVIEW_H

/* system includes */
#include <Addresses/Addresses.h>

/* my includes */
/* (none) */

#include <AddressView/ADPersonView.h>
#include <AddressView/ADSinglePropertyView.h>

#endif /* ADDRESSVIEW_H */
