// Addresses.h (this is -*- ObjC -*-)
// 
// \author: Björn Giesler <giesler@ira.uka.de>
// 
// Address Book Framework for GNUstep
// 
// $Author: bjoern $
// $Locker:  $
// $Revision: 1.2 $
// $Date: 2004/06/14 05:48:08 $

#ifndef CONTACTS_H
#define CONTACTS_H

#include <Addresses/ADAddressBook.h>
#include <Addresses/ADConverter.h>
#include <Addresses/ADEnvelopeAddressBook.h>
#include <Addresses/ADGlobals.h>
#include <Addresses/ADGroup.h>
#include <Addresses/ADImageLoading.h>
#include <Addresses/ADLocalAddressBook.h>
#include <Addresses/ADMultiValue.h>
#include <Addresses/ADPListConverter.h>
#include <Addresses/ADPlugin.h>
#include <Addresses/ADPublicAddressBook.h>
#include <Addresses/ADPerson.h>
#include <Addresses/ADRecord.h>
#include <Addresses/ADSearchElement.h>
#include <Addresses/ADTypedefs.h>
#include <Addresses/ADVCFConverter.h>

#endif /* CONTACTS_H */
